/*
	main.c
	main of wavelet image resize program.
	Copyright(c) & programmed 2004 by NAKATA H. (webmaster@nakata-jp.org)
	$Header: /home/nakata/programs/photo/wavelet2/RCS/main.c,v 1.1 2004/05/27 08:06:54 nakata Exp $

	this program comes with ABSOLUTELY NO WARRANTY.
	This is free software. You can redistribute, customize and sell it.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "wavelet2.h"

INT_TYPE	image_red[IMAGE_WIDTH * IMAGE_HEIGHT],
		image_green[IMAGE_WIDTH * IMAGE_HEIGHT],
		image_blue[IMAGE_WIDTH * IMAGE_HEIGHT];
int		width, height;

static void	read_bmp( FILE * );

extern void	win_init( int *, char **[] );
extern void	win_start( void );
extern int	roi_flag;

/* main section of this program	---------------------------------------------*/
int	main( int argc, char *argv[] )
{
	FILE	*fp;

	win_init( &argc, &argv );

	if ( argc == 2 ) {
		if ( (fp = fopen(argv[1], "rb")) == NULL ) {
			fprintf(stderr, "fopen(%s) failed.\n", argv[1] );
			exit( 1 );
		}
		read_bmp( fp );
		fclose( fp );
	} else if ( argc != 1 ) {
		fprintf(stderr, "usage: %s [input_BMP_file]\n", argv[0] );
		exit( 1 );
	}

	width = IMAGE_WIDTH;
	height = IMAGE_HEIGHT;

	win_start();

	return ( 0 );
}

/* Read the image from a 24-bit BMP file.	-----------------------------*/
static void	read_bmp( FILE *ifp )
{
	char	buff[2];
	short	s;
	long	l, offset;
	INT_TYPE	*p_red, *p_green, *p_blue;
	int	x, y, pudlen, red, green, blue;

	if ( fread( buff, 1, 2, ifp ) < 2 ) {
		fprintf(stderr, "read_bmp() fread() failed.\n");
		exit ( 1 );
	}
	if ( strncmp( buff, "BM", 2 ) != 0 ) {
		fprintf( stderr, "this is not BMP file.\n");
		exit ( 1 );
	}

	/* file size */
	if ( fread( &l, sizeof(l), 1, ifp ) < 1 ) {
		fprintf(stderr, "read_bmp() fread() failed.\n");
		exit( 1 );
	}

	/* reserved */
	if ( fread( &l, sizeof(l), 1, ifp ) < 1 ) {
		fprintf(stderr, "read_bmp() fread() failed.\n");
		exit( 1 );
	}

	/* offset */
	if ( fread( &l, sizeof(l), 1, ifp ) < 1 ) {
		fprintf(stderr, "read_bmp() fread() failed.\n");
		exit( 1 );
	}
	offset = itohl(l);

	/* header size */
	if ( fread( &l, sizeof(l), 1, ifp ) < 1 ) {
		fprintf(stderr, "read_bmp() fread() failed.\n");
		exit( 1 );
	}

	/* horizontal dots */
	if ( fread( &l, sizeof(l), 1, ifp ) < 1 ) {
		fprintf(stderr, "read_bmp() fread() failed.\n");
		exit( 1 );
	}
	width = itohl(l);

	/* vertical dots */
	if ( fread( &l, sizeof(l), 1, ifp ) < 1 ) {
		fprintf(stderr, "read_bmp() fread() failed.\n");
		exit( 1 );
	}
	height = itohl(l);

	/* plane */
	if ( fread( &s, sizeof(s), 1, ifp ) < 1 ) {
		fprintf(stderr, "read_bmp() fread() failed.\n");
		exit( 1 );
	}
	if ( itohs(s) != 1 ) {
		fprintf(stderr, "this program support only plane=1\n" );
		exit ( 1 );
	}

	/* bits/pixel */
	if ( fread( &s, sizeof(s), 1, ifp ) < 1 ) {
		fprintf(stderr, "read_bmp() fread() failed.\n");
		exit( 1 );
	}
	if ( itohs(s) != 24 ) {
		fprintf(stderr, "this program support only 24bit/plane\n" );
		exit ( 1 );
	}

	if ( fseek( ifp, offset, SEEK_SET ) != 0 ) {
		fprintf(stderr, "read_bmp() seek() error.\n" );
		exit ( 1 );
	}

	pudlen = (width * 3) % 4;
	pudlen = (4 - pudlen) & 3;
	offset = width * 3 + pudlen;
	for ( y = 0; y < height; y++ ) {
		if ( height - y > IMAGE_HEIGHT ) {
			if ( fseek( ifp, offset, SEEK_CUR ) != 0 ) {
				fprintf(stderr, "read_bmp() seek() error.\n" );
				exit ( 1 );
			}
			continue;
		}
		p_blue = image_blue + (height - 1 - y) * IMAGE_WIDTH;
		p_green = image_green + (height - 1 - y) * IMAGE_WIDTH;
		p_red = image_red + (height - 1 - y) * IMAGE_WIDTH;
		for ( x = 0; x < width; x++ ) {
			blue = fgetc( ifp );
			green = fgetc( ifp );
			red = fgetc( ifp );
			if ( x < IMAGE_WIDTH ) {
				*p_blue++ = blue;
				*p_green++ = green;
				*p_red++ = red;
			}
		}
		/* adjust to word align */
		for ( x = 0; x < pudlen; x++ ) {
			fgetc( ifp );
		}
	}
}

#define	HEADER_LENGTH	32
static char	header[HEADER_LENGTH] = "wavelet data by Nakata\0";
static INT_TYPE	*image_buff[] = { image_red, image_green, image_blue, NULL};

/* write wavelet data to a file	---------------------------------------------*/
void	write_wave( const char *file_name )
{
	FILE	*fp;
	short	s;
	int	col, x, y;
	INT_TYPE	*p;

	if ( (fp = fopen(file_name, "wb")) == NULL ) {
		fprintf(stderr,"fopen(%s) failed.\n", file_name );
		return;
	}
	if ( fwrite( header, HEADER_LENGTH, 1, fp ) != 1 ) {
		fprintf(stderr, "fwrite() failed.\n" );
		fclose( fp );
		return;
	}
	for ( col = 0; (p = image_buff[col]) != NULL; col++ ) {
		for ( y = 0; y < IMAGE_HEIGHT; y++ ) {
			for ( x = 0; x < IMAGE_WIDTH; x++ ) {
				s = *p++;
				s = htons(s);
				if ( fwrite( &s, sizeof(short), 1, fp ) != 1 ) {
					fprintf(stderr, "fwrite() failed.\n" );
					fclose( fp );
					return;
				}
			}
		}
	}
	fclose( fp );
}

/* read wavelet data to a file	---------------------------------------------*/
const char	*read_wave( const char *file_name )
{
	FILE	*fp;
	char	buff[HEADER_LENGTH];
	short	s;
	int	col, x, y;
	INT_TYPE	*p;

	if ( (fp = fopen(file_name, "rb")) == NULL ) {
		fprintf(stderr,"fopen(%s) failed.\n", file_name );
		return ("fopen() failed");
	}
	if ( fread( buff, HEADER_LENGTH, 1, fp ) != 1 ) {
		fprintf(stderr, "fread() failed.\n" );
		fclose( fp );
		return ("fread() failed");
	}
	if ( strncmp( buff, header, HEADER_LENGTH ) != 0 ) {
		return ("file is not wavelet data");
	}
	for ( col = 0; (p = image_buff[col]) != NULL; col++ ) {
		for ( y = 0; y < IMAGE_HEIGHT; y++ ) {
			for ( x = 0; x < IMAGE_WIDTH; x++ ) {
				if ( fread( &s, sizeof(short), 1, fp ) != 1 ) {
					fprintf(stderr, "fread() failed.\n" );
					fclose( fp );
					return ("fread() failed");
				}
				*p++ = ntohs(s);
			}
		}
	}
	fclose( fp );
	width = height = TERMINAL_WIDTH;
	roi_flag = TRUE;
	return (NULL);
}
/* End Of File	**************************************************************/
