/*
	wavelet.c
	wavelet section of wavelet image resize program.
	Copyright(c) & programmed 2004 by NAKATA H. (webmaster@nakata-jp.org)
	$Header: /home/nakata/programs/photo/wavelet2/RCS/wavelet.c,v 1.1 2004/05/27 08:06:54 nakata Exp $

	this program comes with ABSOLUTELY NO WARRANTY.
	This is free software. You can redistribute, customize and sell it.
 */

#include <stdio.h>
#include <stdlib.h>
#include "wavelet2.h"

extern INT_TYPE	image_red[], image_green[], image_blue[];
extern int	width, height;

static void	wavelet( INT_TYPE *, int, int );
static void	rwavelet( INT_TYPE *, int, int );

static INT_TYPE	*buff = NULL;
static size_t	buff_size = 0;

/* set ROI function	-----------------------------------------------------*/
void	set_roi()
{
	int	i, j, k, l;
	float	f;

	if ( width != TERMINAL_WIDTH ) {
		return;
	}
	for ( f = 0.7,i = IMAGE_WIDTH / 2; i >= TERMINAL_WIDTH; i /= 2,f *= 0.8 ) {
		j = (int) (f * i);
		for ( k = 0; k < j; k++ ) {
			for ( l = 0; l < j - k; l++ ) {
				image_red[k*IMAGE_WIDTH + i + l] = 0;
				image_red[k*IMAGE_WIDTH + i * 2 - 1 - l] = 0;
				image_red[(i - 1 - k)*IMAGE_WIDTH + i + l] = 0;
				image_red[(i - 1 - k)*IMAGE_WIDTH + i*2 - 1 - l] = 0;
				image_red[(i + k)*IMAGE_WIDTH + l] = 0;
				image_red[(i + k)*IMAGE_WIDTH + i - 1 - l] = 0;
				image_red[(i*2 - 1 - k)*IMAGE_WIDTH + l] = 0;
				image_red[(i*2 - 1 - k)*IMAGE_WIDTH + i - 1 - l] = 0;
				image_red[(i + k)*IMAGE_WIDTH + i + l] = 0;
				image_red[(i + k)*IMAGE_WIDTH + i*2 - 1 - l] = 0;
				image_red[(i*2 - 1 - k)*IMAGE_WIDTH + i + l] = 0;
				image_red[(i*2 - 1 - k)*IMAGE_WIDTH + i*2 - 1 - l] = 0;
				image_green[k*IMAGE_WIDTH + i + l] = 0;
				image_green[k*IMAGE_WIDTH + i * 2 - 1 - l] = 0;
				image_green[(i - 1 - k)*IMAGE_WIDTH + i + l] = 0;
				image_green[(i - 1 - k)*IMAGE_WIDTH + i*2 - 1 - l] = 0;
				image_green[(i + k)*IMAGE_WIDTH + l] = 0;
				image_green[(i + k)*IMAGE_WIDTH + i - 1 - l] = 0;
				image_green[(i*2 - 1 - k)*IMAGE_WIDTH + l] = 0;
				image_green[(i*2 - 1 - k)*IMAGE_WIDTH + i - 1 - l] = 0;
				image_green[(i + k)*IMAGE_WIDTH + i + l] = 0;
				image_green[(i + k)*IMAGE_WIDTH + i*2 - 1 - l] = 0;
				image_green[(i*2 - 1 - k)*IMAGE_WIDTH + i + l] = 0;
				image_green[(i*2 - 1 - k)*IMAGE_WIDTH + i*2 - 1 - l] = 0;
				image_blue[k*IMAGE_WIDTH + i + l] = 0;
				image_blue[k*IMAGE_WIDTH + i * 2 - 1 - l] = 0;
				image_blue[(i - 1 - k)*IMAGE_WIDTH + i + l] = 0;
				image_blue[(i - 1 - k)*IMAGE_WIDTH + i*2 - 1 - l] = 0;
				image_blue[(i + k)*IMAGE_WIDTH + l] = 0;
				image_blue[(i + k)*IMAGE_WIDTH + i - 1 - l] = 0;
				image_blue[(i*2 - 1 - k)*IMAGE_WIDTH + l] = 0;
				image_blue[(i*2 - 1 - k)*IMAGE_WIDTH + i - 1 - l] = 0;
				image_blue[(i + k)*IMAGE_WIDTH + i + l] = 0;
				image_blue[(i + k)*IMAGE_WIDTH + i*2 - 1 - l] = 0;
				image_blue[(i*2 - 1 - k)*IMAGE_WIDTH + i + l] = 0;
				image_blue[(i*2 - 1 - k)*IMAGE_WIDTH + i*2 - 1 - l] = 0;
			}
		}
	}
}
		
/* halfnize function	-----------------------------------------------------*/
int	halfnize()
{
	int	i;

	if ( width == TERMINAL_WIDTH ) {
		return ( FALSE );
	}
	if ( width == height ) {
		for ( i = 0; i < width; i++ ) {
			wavelet( image_red + i, height, IMAGE_WIDTH );
			wavelet( image_green + i, height, IMAGE_WIDTH );
			wavelet( image_blue + i, height, IMAGE_WIDTH );
		}
		height /= 2;
	} else {
		for ( i = 0; i < height * 2; i++ ) {
			wavelet( image_red + i * IMAGE_WIDTH, width, 1 );
			wavelet( image_green + i * IMAGE_WIDTH, width, 1 );
			wavelet( image_blue + i * IMAGE_WIDTH, width, 1 );
		}
		width /= 2;
	}
	return (TRUE);
}

/* twice function	-----------------------------------------------------*/
int	twice()
{
	int	i;

	if ( height == IMAGE_HEIGHT ) {
		return ( FALSE );
	}
	if ( width == height ) {
		for ( i = 0; i < height * 2; i++ ) {
			rwavelet( image_red + i * IMAGE_WIDTH, width, 1 );
			rwavelet( image_green + i * IMAGE_WIDTH, width, 1 );
			rwavelet( image_blue + i * IMAGE_WIDTH, width, 1 );
		}
		width *= 2;
	} else {
		for ( i = 0; i < width; i++ ) {
			rwavelet( image_red + i, height, IMAGE_WIDTH );
			rwavelet( image_green + i, height, IMAGE_WIDTH );
			rwavelet( image_blue + i, height, IMAGE_WIDTH );
		}
		height *= 2;
	}
	return (TRUE);
}

#define	BEFORE_SHIFT	2	/* start index i0 == 0 (even) */
#define	AFTER_SHIFT	2	/* end index i1 is even */

/* wavelet function	-----------------------------------------------------*/
static void	wavelet( INT_TYPE *source, int length, int skip )
{
	int		i;
	size_t		size;
	INT_TYPE	*p;

	size = sizeof(INT_TYPE) * (BEFORE_SHIFT + length + AFTER_SHIFT);
	if ( buff == NULL ) {
		buff = (INT_TYPE *)malloc(size);
		buff_size = size;
	} else if ( buff_size < size ) {
		buff = (INT_TYPE *)realloc(buff, size);
		buff_size = size;
	}
	if ( buff == NULL ) {
		fprintf(stderr, "malloc() failed.\n" );
		exit( 1 );
	}
	p = source;
	for ( i = 0; i < length; i++ ) {
		buff[BEFORE_SHIFT + i] = *p;
		p += skip;
	}

	/* periodic symmetric extension */
	for ( i = 0; i < BEFORE_SHIFT; i++ ) {
		buff[i] = buff[BEFORE_SHIFT * 2 - i];
	}
	for ( i = 0; i < AFTER_SHIFT; i++ ) {
		buff[BEFORE_SHIFT + length + i]
			= buff[BEFORE_SHIFT + length - 2 - i];
	}

	for ( i = 0; i < length / 2 + 1; i++ ) {
		buff[2 * i + 1] -= ( buff[2 * i] + buff[2 * i + 2] ) / 2;
	}
	for ( i = 1; i < length / 2 + 1; i++ ) {
		buff[2 * i] += ( buff[2 * i - 1] + buff[2 * i + 1] + 2 ) / 4;
	}

	p = source;
	for ( i = 0; i < (length + 1)/2; i++ ) {
		*p = buff[BEFORE_SHIFT + 2 * i];
		p += skip;
	}
	for ( i = 0; i < (length + 1)/2; i++ ) {
		*p = buff[BEFORE_SHIFT + 2 * i + 1];
		p += skip;
	}
}
#undef	BEFORE_SHIFT
#undef	AFTER_SHIFT

#define	BEFORE_SHIFT	1
#define	AFTER_SHIFT	2
/* reverse wavelet function	---------------------------------------------*/
static void	rwavelet( INT_TYPE *source, int length, int skip )
{
	int		i;
	size_t		size;
	INT_TYPE	*p;

	size = sizeof(INT_TYPE) * (BEFORE_SHIFT + length * 2 + AFTER_SHIFT);
	if ( buff == NULL ) {
		buff = (INT_TYPE *)malloc(size);
		buff_size = size;
	} else if ( buff_size < size ) {
		buff = (INT_TYPE *)realloc(buff, size);
		buff_size = size;
	}
	if ( buff == NULL ) {
		fprintf(stderr, "malloc() failed.\n" );
		exit( 1 );
	}

	p = source;
	for ( i = 0; i < length; i++ ) {
		buff[BEFORE_SHIFT + 2 * i] = *p;
		p += skip;
	}
	for ( i = 0; i < length; i++ ) {
		buff[BEFORE_SHIFT + 2 * i + 1] = *p;
		p += skip;
	}

	/* periodic symmetric extension */
	for ( i = 0; i < BEFORE_SHIFT; i++ ) {
		buff[i] = buff[BEFORE_SHIFT * 2 - i];
	}
	for ( i = 0; i < AFTER_SHIFT; i++ ) {
		buff[BEFORE_SHIFT + length * 2 + i]
			= buff[BEFORE_SHIFT + length * 2 - 2 - i];
	}

	for ( i = 0; i < length + 1; i++ ) {
		buff[2 * i + 1] -= ( buff[2 * i] + buff[2 * i + 2] + 2 ) / 4;
	}
	for ( i = 0; i < length; i++ ) {
		buff[2 * i + 2] += ( buff[2 * i + 1] + buff[2 * i + 3] ) / 2;
	}

	p = source;
	for ( i = 0; i < length * 2; i++ ) {
		*p = buff[BEFORE_SHIFT + i];
		p += skip;
	}
}
/* End Of File	**************************************************************/
