/*
	window.c
	window of wavelet image resize program.
	Copyright(c) & programmed 2004 by NAKATA H. (webmaster@nakata-jp.org)
	$Header: /home/nakata/programs/photo/wavelet2/RCS/window.c,v 1.3 2006/03/07 12:31:30 nakata Exp $

	this program comes with ABSOLUTELY NO WARRANTY.
	This is free software. You can redistribute, customize and sell it.
 */

#include <gtk/gtk.h>
#include "wavelet2.h"

#define	BUTTON_FORWARD_ALL	0	/* button number */
#define	BUTTON_FORWARD_STEP	1	/* button number */
#define	BUTTON_BACKWARD_STEP	2	/* button number */
#define	BUTTON_BACKWARD_ALL	3	/* button number */
#define	BUTTON_ROI		4	/* button number */
#define	BUTTON_SAVE		5	/* button number */
#define	BUTTON_LOAD		6	/* button number */
#define	BUTTONS			7	/* button count */

extern INT_TYPE	image_red[], image_green[], image_blue[];
extern int	width, height;
extern int	halfnize( void );
extern int	twice( void );
extern void	set_roi( void );
extern void	write_wave( const char * );
extern const char	*read_wave( const char * );

int	roi_flag = TRUE;

static void	win_sync( void );
static void	cb_forward_all( GtkWidget *, gpointer );
static void	cb_forward_step( GtkWidget *, gpointer );
static void	cb_backward_all( GtkWidget *, gpointer );
static void	cb_backward_step( GtkWidget *, gpointer );
static void	cb_roi( GtkWidget *, gpointer );
static void	cb_save( GtkWidget *, gpointer );
static void	cb_save2( GtkWidget *, gpointer );
static void	cb_load( GtkWidget *, gpointer );
static void	cb_load2( GtkWidget *, gpointer );
static void	cb_close( GtkWidget *, gpointer );
static gint     expose_event(GtkWidget *, GdkEventExpose *);
static void     quit( void );
static void	button_show_hide( void );

static GtkWidget	*drawing_area, *button[BUTTONS];
static GdkPixmap	*pixmap = NULL;

/* initialize window	-----------------------------------------------------*/
void	win_init( int *p_argc, char **p_argv[] )
{
	gtk_init( p_argc, p_argv );
}

/* go main loop -------------------------------------------------------------*/
void	win_start()
{
	GtkWidget	*window, *table;

	window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_window_set_title(GTK_WINDOW(window), MYNAME);
	gtk_widget_show(window);

	table = gtk_table_new( BUTTONS, 1, FALSE );
	gtk_container_add( GTK_CONTAINER(window), table );

	drawing_area = gtk_drawing_area_new();
	gtk_drawing_area_size( GTK_DRAWING_AREA(drawing_area),
				IMAGE_WIDTH, IMAGE_HEIGHT );
	gtk_table_attach_defaults( GTK_TABLE(table), drawing_area,
				0, 1, 0, BUTTONS );
	gtk_widget_show(drawing_area);

	button[BUTTON_FORWARD_ALL] = gtk_button_new_with_label("forward to end");
	gtk_signal_connect( GTK_OBJECT(button[BUTTON_FORWARD_ALL]), "clicked",
			    GTK_SIGNAL_FUNC(cb_forward_all), NULL );
	gtk_table_attach_defaults( GTK_TABLE(table),
					button[BUTTON_FORWARD_ALL],
					1, 2, BUTTON_FORWARD_ALL,
					BUTTON_FORWARD_ALL + 1 );
	gtk_widget_show(button[BUTTON_FORWARD_ALL]);

	button[BUTTON_FORWARD_STEP] = gtk_button_new_with_label("step forward");
	gtk_signal_connect( GTK_OBJECT(button[BUTTON_FORWARD_STEP]), "clicked",
			    GTK_SIGNAL_FUNC(cb_forward_step), NULL );
	gtk_table_attach_defaults( GTK_TABLE(table),
					button[BUTTON_FORWARD_STEP],
					1, 2, BUTTON_FORWARD_STEP,
					BUTTON_FORWARD_STEP + 1 );
	gtk_widget_show(button[BUTTON_FORWARD_STEP]);

	button[BUTTON_BACKWARD_ALL] = gtk_button_new_with_label("backward to start");
	gtk_signal_connect( GTK_OBJECT(button[BUTTON_BACKWARD_ALL]), "clicked",
			    GTK_SIGNAL_FUNC(cb_backward_all), NULL );
	gtk_table_attach_defaults( GTK_TABLE(table),
					button[BUTTON_BACKWARD_ALL],
					1, 2, BUTTON_BACKWARD_ALL,
					BUTTON_BACKWARD_ALL + 1 );
	gtk_widget_show(button[BUTTON_BACKWARD_ALL]);

	button[BUTTON_BACKWARD_STEP] = gtk_button_new_with_label("step backward");
	gtk_signal_connect( GTK_OBJECT(button[BUTTON_BACKWARD_STEP]), "clicked",
			    GTK_SIGNAL_FUNC(cb_backward_step), NULL );
	gtk_table_attach_defaults( GTK_TABLE(table),
					button[BUTTON_BACKWARD_STEP],
					1, 2, BUTTON_BACKWARD_STEP,
					BUTTON_BACKWARD_STEP + 1 );
	gtk_widget_show(button[BUTTON_BACKWARD_STEP]);

	button[BUTTON_ROI] = gtk_button_new_with_label("set ROI");
	gtk_signal_connect( GTK_OBJECT(button[BUTTON_ROI]), "clicked",
			    GTK_SIGNAL_FUNC(cb_roi), NULL );
	gtk_table_attach_defaults( GTK_TABLE(table),
					button[BUTTON_ROI],
					1, 2, BUTTON_ROI,
					BUTTON_ROI + 1 );
	gtk_widget_show(button[BUTTON_ROI]);

	button[BUTTON_SAVE] = gtk_button_new_with_label("save to file");
	gtk_signal_connect( GTK_OBJECT(button[BUTTON_SAVE]), "clicked",
			    GTK_SIGNAL_FUNC(cb_save), NULL );
	gtk_table_attach_defaults( GTK_TABLE(table),
					button[BUTTON_SAVE],
					1, 2, BUTTON_SAVE,
					BUTTON_SAVE + 1 );
	gtk_widget_show(button[BUTTON_SAVE]);

	button[BUTTON_LOAD] = gtk_button_new_with_label("load from file");
	gtk_signal_connect( GTK_OBJECT(button[BUTTON_LOAD]), "clicked",
			    GTK_SIGNAL_FUNC(cb_load), NULL );
	gtk_table_attach_defaults( GTK_TABLE(table),
					button[BUTTON_LOAD],
					1, 2, BUTTON_LOAD,
					BUTTON_LOAD + 1 );
	gtk_widget_show(button[BUTTON_LOAD]);

	gtk_widget_show(table);
	gtk_signal_connect( GTK_OBJECT(window), "destroy",
				GTK_SIGNAL_FUNC(quit), NULL);
	gtk_signal_connect( GTK_OBJECT(drawing_area), "expose_event",
				(GtkSignalFunc)expose_event, NULL);

	win_sync();
	button_show_hide();

	gtk_main();
}

/*      quit program with window clean up       -----------------------------*/
static void     quit()
{
	gtk_exit(0);
}

#define	ROUND_CHAR(value)	( (value > MAX_UCHAR) ? MAX_UCHAR : \
				  (value < 0 ) ? 0 : value )
/* display iamge buffer	-----------------------------------------------------*/
static void	win_sync()
{
	GdkDrawable	*drawable;
	GdkColor	c;
	GdkGC		*gc;
	int		x, y;
	INT_TYPE	red, green, blue, *p_red, *p_green, *p_blue;

	drawable = drawing_area->window;
	if ( pixmap != NULL ) {
		gdk_pixmap_unref( pixmap );
	}
	pixmap = gdk_pixmap_new(drawable, IMAGE_WIDTH, IMAGE_HEIGHT, -1);
	c.pixel = 0;
	gc = gdk_gc_new(drawable);
	p_red = image_red;
	p_green = image_green;
	p_blue = image_blue;
	for ( y = 0; y < IMAGE_HEIGHT; y++ ) {
		for ( x = 0; x < IMAGE_WIDTH; x++ ) {
			red = *p_red++;
			green = *p_green++;
			blue = *p_blue++;
			if ( x >= width || y >= height ) {
				red += 128;
				green += 128;
				blue += 128;
			}
			c.red = ROUND_CHAR( red ) << 8;
			c.green = ROUND_CHAR( green ) << 8;
			c.blue = ROUND_CHAR( blue ) << 8;
			gdk_color_alloc(gdk_colormap_get_system(), &c);
			gdk_gc_set_foreground(gc, &c);
			gdk_draw_point(pixmap, gc, x, y);
		}
	}
	gdk_draw_pixmap(drawable,
			drawing_area->style->fg_gc[GTK_WIDGET_STATE(drawing_area)],
			pixmap,
			0, 0, 0, 0,
			IMAGE_WIDTH, IMAGE_HEIGHT);
}

/* redraw window        -----------------------------------------------------*/
static gint     expose_event(GtkWidget *widget, GdkEventExpose *event)
{
	gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
			pixmap, event->area.x, event->area.y,
			event->area.x, event->area.y,
			event->area.width, event->area.height);
											return (FALSE);
}

/* set button show/hide status	---------------------------------------------*/
static void	button_show_hide()
{
	if ( width == TERMINAL_WIDTH ) {
		gtk_widget_hide(GTK_BIN(button[BUTTON_FORWARD_STEP])->child);
		gtk_widget_hide(GTK_BIN(button[BUTTON_FORWARD_ALL])->child);
		gtk_widget_show(GTK_BIN(button[BUTTON_SAVE])->child);
	} else {
		gtk_widget_show(GTK_BIN(button[BUTTON_FORWARD_STEP])->child);
		gtk_widget_show(GTK_BIN(button[BUTTON_FORWARD_ALL])->child);
		gtk_widget_hide(GTK_BIN(button[BUTTON_SAVE])->child);
	}
	if ( height == IMAGE_HEIGHT ) {
		gtk_widget_hide(GTK_BIN(button[BUTTON_BACKWARD_STEP])->child);
		gtk_widget_hide(GTK_BIN(button[BUTTON_BACKWARD_ALL])->child);
	} else {
		gtk_widget_show(GTK_BIN(button[BUTTON_BACKWARD_STEP])->child);
		gtk_widget_show(GTK_BIN(button[BUTTON_BACKWARD_ALL])->child);
	}
	if ( width == TERMINAL_WIDTH && roi_flag ) {
		gtk_widget_show(GTK_BIN(button[BUTTON_ROI])->child);
	} else {
		gtk_widget_hide(GTK_BIN(button[BUTTON_ROI])->child);
	}
}

/* call back of calculate to end button	-------------------------------------*/
static void	cb_forward_all( GtkWidget *widget, gpointer ptr )
{
	while ( halfnize() )
		;	/* empty loop body */
	win_sync();
	button_show_hide();
}

/* call back of forward step button	-------------------------------------*/
static void	cb_forward_step( GtkWidget *widget, gpointer ptr )
{
	halfnize();
	win_sync();
	button_show_hide();
}

/* call back of calculate to end button	-------------------------------------*/
static void	cb_backward_all( GtkWidget *widget, gpointer ptr )
{
	while ( twice() )
		;	/* empty loop body */
	win_sync();
	button_show_hide();
}

/* call back of forward step button	-------------------------------------*/
static void	cb_backward_step( GtkWidget *widget, gpointer ptr )
{
	twice();
	win_sync();
	button_show_hide();
}

/* call back of ROI button	---------------------------------------------*/
static void	cb_roi( GtkWidget *widget, gpointer ptr )
{
	set_roi();
	roi_flag = FALSE;
	win_sync();
	button_show_hide();
}

/* call back of save button	---------------------------------------------*/
static void	cb_save( GtkWidget *widget, gpointer ptr )
{
	GtkWidget	*dialog;

	dialog = gtk_file_selection_new( "wavelet save" );
	gtk_window_position( GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE );
	gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(dialog)->cancel_button),
				"clicked", (GtkSignalFunc)cb_close,
				GTK_OBJECT(GTK_FILE_SELECTION(dialog)) );
	gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(dialog)->ok_button),
				"clicked", (GtkSignalFunc)cb_save2,
				GTK_OBJECT(GTK_FILE_SELECTION(dialog)) );
	gtk_widget_show(dialog);
}

/* call back of save dialog	---------------------------------------------*/
static void	cb_save2( GtkWidget *widget, gpointer ptr )
{
	char	*p;

	p = gtk_file_selection_get_filename( GTK_FILE_SELECTION(ptr) );
	if ( p == NULL || strlen(p) == 0 ) {
		return;
	}
	write_wave( p );
	gtk_widget_destroy( ptr );
	win_sync();
	button_show_hide();
}

/* call back of load button	---------------------------------------------*/
static void	cb_load( GtkWidget *widget, gpointer ptr )
{
	GtkWidget	*dialog;

	dialog = gtk_file_selection_new( "wavelet load" );
	gtk_window_position( GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE );
	gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(dialog)->cancel_button),
				"clicked", (GtkSignalFunc)cb_close,
				GTK_OBJECT(GTK_FILE_SELECTION(dialog)) );
	gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(dialog)->ok_button),
				"clicked", (GtkSignalFunc)cb_load2,
				GTK_OBJECT(GTK_FILE_SELECTION(dialog)) );
	gtk_widget_show(dialog);
}

/* call back of load dialog	---------------------------------------------*/
static void	cb_load2( GtkWidget *widget, gpointer ptr )
{
	const char	*p;
	GtkWidget	*dialog, *label, *button;

	p = gtk_file_selection_get_filename( GTK_FILE_SELECTION(ptr) );
	if ( p == NULL || strlen(p) == 0 ) {
		return;
	}
	if ( (p = read_wave( p )) == NULL ) {
		gtk_widget_destroy( ptr );
		win_sync();
		button_show_hide();
		return;
	}

	gtk_widget_destroy( ptr );
	dialog = gtk_dialog_new();
	gtk_window_position( GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE );
	gtk_signal_connect( GTK_OBJECT(GTK_DIALOG(dialog)), "destroy",
				GTK_SIGNAL_FUNC(cb_close), dialog );
	gtk_window_set_title( GTK_WINDOW(dialog), "confirm" );
	gtk_container_border_width( GTK_CONTAINER(dialog), 5 );
	label = gtk_label_new( p );
	gtk_misc_set_padding( GTK_MISC(label), 10, 10 );
	gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->vbox), label,
				TRUE, TRUE, 0 );
	gtk_widget_show(label);
	button = gtk_button_new_with_label("Confirm");
	gtk_signal_connect( GTK_OBJECT(button), "clicked",
				GTK_SIGNAL_FUNC(cb_close), dialog );
	gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->action_area),
				button, TRUE, TRUE, 0 );
	gtk_widget_show(button);
	gtk_widget_show(dialog);
}

/* call back of close dialog	---------------------------------------------*/
static void	cb_close( GtkWidget *widget, gpointer ptr )
{
	gtk_widget_destroy( GTK_WIDGET(ptr) );
}
/* End Of File	**************************************************************/
